#!/bin/sh

#Package relative definition
PATH_SYSLOGNG_PKG_DIR=/var/packages/SyslogServer
PATH_SYSLOGNG_ETC_DIR=${PATH_SYSLOGNG_PKG_DIR}/etc
PATH_SYSLOGNG_SRC_DIR=${PATH_SYSLOGNG_PKG_DIR}/target

# Config
FILE_SYSLOGNG_SERVER_CONFIG=${PATH_SYSLOGNG_ETC_DIR}/setting.conf
# Template
PATH_SYSLOGNG_TEMPLATE_FOLDER=${PATH_SYSLOGNG_SRC_DIR}/etc/template


#Service relative definition
PATH_SYSLOGNG_CONF=/usr/syno/etc/synosyslogserver
PATH_SYSLOGNG_CONF_FOLDER=${PATH_SYSLOGNG_CONF}/syslog.d
FILE_SYNOSYSLOG_CONF=${PATH_SYSLOGNG_CONF_FOLDER}/syno.conf


FILE_SYSLOGNG_SERVER_PID=/var/run/syslogng_server.pid
FILE_SYSLOGD_PID=/var/run/syslogd.pid
FILE_DISPATCHD_PID=/var/run/logdispatchd.pid

FILE_SYNOSYSLOG_SQL=${PATH_SYSLOGNG_PKG_DIR}/target/scripts/loginfo2.sql #Use new schema

#Global variables
SYSLOGNG_SERVER_ENABLE=0
SYSLOGNG_SERVER_PORT=1234
SYSLOGNG_SERVER_TRANS_MODE=tcp
SYSLOGNG_SERVER_SSL=0
SYSLOGNG_SERVER_IETF_PORT=1235
SYSLOGNG_SERVER_IETF_TRANS_MODE=tcp
SYSLOGNG_SERVER_IETF_SSL=0
SYSLOGNG_SERVER_LOG_ROTATE_TIME=0
SYSLOGNG_SERVER_MAIL_ENABLE=0
SYSLOGNG_SERVER_MAIL_FILETER=""
SYSLOGNG_SERVER_DB_LOC=""
SYSLOGNG_SERVER_DB_PATH=""
SYSLOGNG_SERVER_DB_PATH_TMP=""
SYSLOGNG_SERVER_CA_KEY_PATH="/usr/syno/etc/ssl/ssl.key/server.key"
SYSLOGNG_SERVER_CA_FILE_PATH="/usr/syno/etc/ssl/ssl.crt/server.crt"
SYSLOGNG_SERVER_INTER_FILETER=""

PROCESS_ALIVE=0
PROCESS_PID=0

SyslogSettingGet(){
    if [ -f "${FILE_SYSLOGNG_SERVER_CONFIG}" ]; then
	SYSLOGNG_SERVER_ENABLE=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_enable_2`
	SYSLOGNG_SERVER_PORT=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_port`
	SYSLOGNG_SERVER_TRANS_MODE=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_trans`
	SYSLOGNG_SERVER_IETF_SSL=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ietf_ssl`
	SYSLOGNG_SERVER_IETF_PORT=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ietf_port`
	SYSLOGNG_SERVER_IETF_TRANS_MODE=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ietf_trans`
	SYSLOGNG_SERVER_SSL=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ssl`
	SYSLOGNG_SERVER_LOG_ROTATE_TIME=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_log_rotate_time`
	SYSLOGNG_SERVER_MAIL_ENABLE=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_enable_mail`
	SYSLOGNG_SERVER_MAIL_FILETER=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_mail_filter`
	SYSLOGNG_SERVER_DB_LOC=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_db_loc`
	SYSLOGNG_SERVER_DB_PATH=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_db_path`
	SYSLOGNG_SERVER_CA_KEY_PATH=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ca_key`
	SYSLOGNG_SERVER_CA_FILE_PATH=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_ca_file`
	SYSLOGNG_SERVER_INTER_FILETER=`/bin/get_key_value ${FILE_SYSLOGNG_SERVER_CONFIG} server_internal_filter`

        SYSLOGNG_SERVER_DB_PATH_TMP=${SYSLOGNG_SERVER_DB_PATH}".tmp."$$
    else
	echo "Syslog setting file does not exist!";
	exit 1;
    fi
}

CheckIfDaemonAlive(){
    PROCESS_ALIVE=0
    if [ -f "$1" ]; then
	PROCESS_PID=`cat $1`
	kill -0 ${PROCESS_PID}
        if [ "0" = "$?" ]; then
	    PROCESS_ALIVE=1
	fi
    fi
}

SourceProcess(){
    #Copy syslog system conf
    cp -rf ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/system.conf ${PATH_SYSLOGNG_CONF}

    if [ "tcp" = "${SYSLOGNG_SERVER_TRANS_MODE}" ]; then
	if [ "1" = "${SYSLOGNG_SERVER_SSL}" ]; then
	    TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_net_enc.template
	    cat $TEMPLATE_FILE | sed -e "s/__SYNO_PORT__/${SYSLOGNG_SERVER_PORT}/" | sed -e "s=__SYNO_CA_KEY_PATH__=${SYSLOGNG_SERVER_CA_KEY_PATH}=" | sed -e "s=__SYNO_CA_FILE_PATH__=${SYSLOGNG_SERVER_CA_FILE_PATH}=" >> ${FILE_SYNOSYSLOG_CONF}
	else
	    TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_net.template
	    cat $TEMPLATE_FILE | sed -e "s/__SYNO_PORT__/${SYSLOGNG_SERVER_PORT}/" >> ${FILE_SYNOSYSLOG_CONF}
	fi
    else
	TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_net_udp.template
	cat $TEMPLATE_FILE | sed -e "s/__SYNO_PORT__/${SYSLOGNG_SERVER_PORT}/" >> ${FILE_SYNOSYSLOG_CONF}
    fi
}

IETFSourceProcess(){
    if [ "tcp" = "${SYSLOGNG_SERVER_IETF_TRANS_MODE}" ]; then
	if [ "1" = "${SYSLOGNG_IETF_SERVER_SSL}" ]; then
	    TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_ietf_enc.template
	    cat $TEMPLATE_FILE | sed -e "s/__SYNO_IETF_PORT__/${SYSLOGNG_SERVER_IETF_PORT}/" | sed -e "s=__SYNO_CA_KEY_PATH__=${SYSLOGNG_SERVER_CA_KEY_PATH}=" | sed -e "s=__SYNO_CA_FILE_PATH__=${SYSLOGNG_SERVER_CA_FILE_PATH}=" >> ${FILE_SYNOSYSLOG_CONF}
	else
	    TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_ietf.template
	    cat $TEMPLATE_FILE | sed -e "s/__SYNO_IETF_PORT__/${SYSLOGNG_SERVER_IETF_PORT}/" >> ${FILE_SYNOSYSLOG_CONF}
	fi
    else
	TEMPLATE_FILE=${PATH_SYSLOGNG_TEMPLATE_FOLDER}/src_ietf_udp.template
	cat $TEMPLATE_FILE | sed -e "s/__SYNO_IETF_PORT__/${SYSLOGNG_SERVER_IETF_PORT}/" >> ${FILE_SYNOSYSLOG_CONF}
    fi
}

MailDestProcess(){
    cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/dest_mail.template >> ${FILE_SYNOSYSLOG_CONF}
}

DestProcess(){
    cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/dest_db.template | sed -e "s=__SYNO_DB_PATH__=${LOGDB_PATH}=" >> ${FILE_SYNOSYSLOG_CONF}
    MailDestProcess;
}

MailLogProcess(){
    if [ ! -z ${SYSLOGNG_PRIOR_STRING} ]; then
	cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/log_send_mail.template | sed -e "s=_SYNO_FILTER_=filter(f_syno_mail);=g" >> ${FILE_SYNOSYSLOG_CONF}
    else
	cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/log_send_mail.template | sed -e "s/_SYNO_FILTER_//g" >> ${FILE_SYNOSYSLOG_CONF}
    fi

}

NetLogProcess(){
    cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/log_net_to_db.template >> ${FILE_SYNOSYSLOG_CONF}
}

LogProcess(){
    NetLogProcess;
    MailLogProcess;
}

MailFilterProcess(){
    if [ "1" != "${SYSLOGNG_SERVER_MAIL_ENABLE}" ]; then
	return;
    fi 

    #1. Get mail level setting from setting.conf
    #2. Compose of filter string for syslog-ng
    #3. Generate log conf file for mailing
    SYSLOGNG_PRIOR_LIST=`echo ${SYSLOGNG_SERVER_MAIL_FILETER} | sed 's/,/ /g'`
    SYSLOGNG_PRIOR_STRING=""
    for prio in $SYSLOGNG_PRIOR_LIST 
    do
	if [ ! -z "${prio}" ]; then
	    if [ -z ${SYSLOGNG_PRIOR_STRING} ]; then
		SYSLOGNG_PRIOR_STRING="${prio}"
	    else
		SYSLOGNG_PRIOR_STRING="${SYSLOGNG_PRIOR_STRING},${prio}"
	    fi
	fi
    done
    
    if [ ! -z ${SYSLOGNG_PRIOR_STRING} ]; then
	cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/filter_prior_mail.template | sed -e "s/_SYNO_PRIORITY_/${SYSLOGNG_PRIOR_STRING}/g" >> ${FILE_SYNOSYSLOG_CONF}
    fi
}

InternalFilterProcess(){
    #1. Get internal level setting from setting.conf
    #2. Compose of filter string for syslog-ng
    #3. Generate log conf file for mailing
    SYSLOGNG_INTERNAL_PRIOR_LIST=`echo ${SYSLOGNG_SERVER_INTER_FILETER} | sed 's/,/ /g'`
    SYSLOGNG_INTERNAL_PRIOR_STRING=""
    for prio in $SYSLOGNG_INTERNAL_PRIOR_LIST 
    do
	if [ -n "${prio}" ]; then
	    if [ -z ${SYSLOGNG_INTERNAL_PRIOR_STRING} ]; then
		SYSLOGNG_INTERNAL_PRIOR_STRING="${prio}"
	    else
		SYSLOGNG_INTERNAL_PRIOR_STRING="${SYSLOGNG_INTERNAL_PRIOR_STRING},${prio}"
	    fi
	fi
    done
    
    if [ -n ${SYSLOGNG_INTERNAL_PRIOR_STRING} ]; then
	cat ${PATH_SYSLOGNG_TEMPLATE_FOLDER}/filter_prior_internal.template | sed -e "s/_SYNO_PRIORITY_/${SYSLOGNG_INTERNAL_PRIOR_STRING}/g" >> ${FILE_SYNOSYSLOG_CONF}
    fi
}

FilterProcess(){
    InternalFilterProcess;
    MailFilterProcess;
}

SyslogngStart(){
    /sbin/syslog-ng --module-path=/lib/syslogmod --pidfile=${FILE_SYSLOGNG_SERVER_PID} --cfgfile=${PATH_SYSLOGNG_CONF}/system.conf --persist-file="/tmp/syslog-ng-server.persist" --control="/tmp/syslog-ng-server.ctl"
}

LogDispatchStart(){
    ${PATH_SYSLOGNG_SRC_DIR}/sbin/logdispatchd
}

ConfDataClear(){
    rm -rf ${PATH_SYSLOGNG_CONF_FOLDER} > /dev/null 2>&1
    rm -rf ${PATH_SYSLOGNG_CONF}/system.conf > /dev/null 2>&1
}

ConfFolderCreate(){
    mkdir -p ${PATH_SYSLOGNG_CONF_FOLDER}
}

ShareExistCheck(){
	SHARE_EXIST=0
	SHARE_PATH=`/usr/syno/sbin/synoshare --get "${SYSLOGNG_SERVER_DB_LOC}" | grep "Path ......" | sed -e 's/^.*\[\(.*\)\]/\1/g'`
	if [ -n "${SHARE_PATH}" ]; then
		SHARE_EXIST=1
		LOGDB_PATH="${SHARE_PATH}/.SYNOSYSLOGDB"
		if [ -d "${SHARE_PATH}" ]; then
			if [ "$LOGDB_PATH" = "$SYSLOGNG_SERVER_DB_PATH" ]; then
				if [ ! -f "$SYSLOGNG_SERVER_DB_PATH" ]; then
					/usr/syno/bin/sqlite3 "$SYSLOGNG_SERVER_DB_PATH_TMP" < "$FILE_SYNOSYSLOG_SQL" > /dev/null 2>&1
                                        /usr/syno/bin/sqlite3 "$SYSLOGNG_SERVER_DB_PATH_TMP" "PRAGMA journal_mode=wal;" > /dev/null 2>&1
                                        mv ${SYSLOGNG_SERVER_DB_PATH_TMP} ${SYSLOGNG_SERVER_DB_PATH}
				fi
			fi
		fi
	fi

	LOGDB_PATH="$SYSLOGNG_SERVER_DB_PATH"
}

ProcStopCheck(){
    COUNT=0;

    # Check process is stopped or not (20 sec for max)
    CheckIfDaemonAlive $1
    while [ "1" = "$PROCESS_ALIVE" ]
    do
        # Force to terminate the process
        if [ ${COUNT} -eq 20 ]; then
            logger -p 3 "$1 cannot stop within 20 seconds, kill it in force"
            PROCESS_PID=`cat $1`
            kill -9 ${PROCESS_PID}
        fi

        # Check again
        sleep 1;
        COUNT=$((COUNT+1))
        CheckIfDaemonAlive $1
    done
}

#At begining, acquire all server settings
SyslogSettingGet;

case $1 in
    start)
	echo "Syslog-ng server start..."

	#Check if server is enable
	case ${SYSLOGNG_SERVER_ENABLE} in
	[Yy][Ee][Ss])
		#Restart syslog-ng daemon
		CheckIfDaemonAlive ${FILE_SYSLOGNG_SERVER_PID}
		if [ "1" = "$PROCESS_ALIVE" ]; then
			kill ${PROCESS_PID}
			sleep 1;
		fi
                ProcStopCheck ${FILE_SYSLOGNG_SERVER_PID}

		ShareExistCheck;
		if [ "0" = "${SHARE_EXIST}" ]; then 
			logger -p 3 "Syslog-ng has no usable share!"
		fi
		ConfDataClear;
		ConfFolderCreate;
		SourceProcess;
		IETFSourceProcess;
		FilterProcess;
		DestProcess;
		LogProcess;
		SyslogngStart;

                #Restart log dispatch daemon
                CheckIfDaemonAlive ${FILE_DISPATCHD_PID}
		if [ "1" = "$PROCESS_ALIVE" ]; then
			kill ${PROCESS_PID}
			sleep 1;
		fi
                ProcStopCheck ${FILE_DISPATCHD_PID}

		LogDispatchStart;
	;;
	*)
	    exit 0;
	;;
	esac
    ;;
    stop)
	echo "Syslog-ng server stop..."

        #Stop syslog-ng daemon
	CheckIfDaemonAlive ${FILE_SYSLOGNG_SERVER_PID}
	if [ "1" = "$PROCESS_ALIVE" ]; then
	    kill ${PROCESS_PID}
	    sleep 1;
	fi
        ProcStopCheck ${FILE_SYSLOGNG_SERVER_PID}
	
        #Stop log dispatch daemon
        CheckIfDaemonAlive ${FILE_DISPATCHD_PID}
        if [ "1" = "$PROCESS_ALIVE" ]; then
                kill ${PROCESS_PID}
                sleep 1;
        fi
        ProcStopCheck ${FILE_DISPATCHD_PID}

	ConfDataClear;
    ;;
    restart)
	echo "Syslog-ng server restart..."

	#Check if server is enable
	case ${SYSLOGNG_SERVER_ENABLE} in
	[Yy][Ee][Ss])
            #Restart syslog-ng daemon
	    ConfDataClear;
	    ConfFolderCreate;
	    ShareExistCheck;
	    if [ "0" = "${SHARE_EXIST}" ]; then 
		logger -p 3 "Syslog-ng has no usable share!"
	    fi

	    SourceProcess;
	    IETFSourceProcess;
	    FilterProcess;
	    DestProcess;
	    LogProcess;
	    CheckIfDaemonAlive ${FILE_SYSLOGNG_SERVER_PID};
	    if [ "1" = "${PROCESS_ALIVE}" ]; then
		kill -HUP ${PROCESS_PID}
	    else
		SyslogngStart; 
	    fi

            #Restart log dispatch daemon
            CheckIfDaemonAlive ${FILE_DISPATCHD_PID}
            if [ "1" = "$PROCESS_ALIVE" ]; then
                    kill ${PROCESS_PID}
                    sleep 1;
            fi
            ProcStopCheck ${FILE_DISPATCHD_PID}

            LogDispatchStart;
	;;
	*)
	    exit 0
	;;
	esac
    ;;
    *)
	echo "Usage: $0 start|stop|restart"
    ;;
esac

